// $Header: //depot/acad/develop/global/src/arxsdk/modules/AsdkHlrApi/Samples/HlrArxCustom/Pie/2dsector.cpp#5 $
// $Change: 293540 $ $DateTime: 2012/01/03 10:55:53 $ $Author: patti $
// $NoKeywords: $ 
//
//////////////////////////////////////////////////////////////////////////////
//
//  Copyright 2012 Autodesk, Inc.  All rights reserved.
//
//  Use of this software is subject to the terms of the Autodesk license 
//  agreement provided at the time of installation or download, or which 
//  otherwise accompanies this software in either electronic or hard copy form.   
//
//////////////////////////////////////////////////////////////////////////////


#include <math.h>

#include "acgi.h"
#include "geassign.h"

#include "dbid.h"
#include "dbmain.h"
#include "dbproxy.h"
#include "dbsymtb.h"
#include "rxboiler.h"

#include "tools.h"
#include "2dsector.h"
ACRX_DXF_DEFINE_MEMBERS (Asdk2dSector, AcDbEntity,
                         AcDb::kDHL_CURRENT, AcDb::kMReleaseCurrent, 
                         AcDbProxyEntity::kAllAllowedBits, ASDK2DSECTOR, AsdkPie
                        ) ;

//-----------------------------------------------------------------------------
Asdk2dSector::Asdk2dSector ()
: m_center (0, 0, 0), m_radius (1), m_start (0), m_stop (PI), m_normal (0, 0, 1) {
}

Asdk2dSector::~Asdk2dSector () {
}

AcGePoint3d Asdk2dSector::center () const {
	assertReadEnabled () ;
	return (m_center) ;
}

void Asdk2dSector::setCenter (const AcGePoint3d &pt) {
	assertWriteEnabled () ;
	m_center =pt ;
}

double Asdk2dSector::radius () const {
	assertReadEnabled () ;
	return (m_radius) ;
}

void Asdk2dSector::setRadius (const double rad) {
	assertWriteEnabled () ;
	m_radius =rad ;
}

double Asdk2dSector::start () const {
	assertReadEnabled () ;
	return (m_start) ;
}

void Asdk2dSector::setStart (const double start) {
	assertWriteEnabled () ;
	m_start =start ;
}

double Asdk2dSector::stop () const {
	assertReadEnabled () ;
	return (m_stop) ;
}

void Asdk2dSector::setStop (const double stop) {
	assertWriteEnabled () ;
	m_stop =stop ;
}

AcGeVector3d Asdk2dSector::normal () const {
	assertReadEnabled () ;
	return (m_normal) ;
}

void Asdk2dSector::setNormal (const AcGeVector3d &vt) {
	assertWriteEnabled () ;
	m_normal =vt ;
}

Acad::ErrorStatus Asdk2dSector::getVertices (AcGePoint3dArray &vertexArray, bool bFlag /*=false*/) const {
    assertReadEnabled () ;

	vertexArray.append (m_center) ;

	AcGeVector3d start (cos (m_start), sin (m_start), 0) ;
	start *=m_radius ;
	ucs2wcs (m_normal, start) ;
	vertexArray.append (m_center+start) ;
	vertexArray.append (m_center+start/2) ;

	AcGeVector3d stop (cos (m_stop), sin (m_stop), 0) ;
	stop *=m_radius ;
	ucs2wcs (m_normal, stop) ;
	vertexArray.append (m_center+stop) ;
	vertexArray.append (m_center+stop/2) ;

	double ang =m_start + (m_stop < m_start ? m_stop + 2*PI : m_stop) ;
	AcGeVector3d mid (cos (ang/2), sin (ang/2), 0) ;
	mid *=m_radius ;
	ucs2wcs (m_normal, mid) ;
	vertexArray.append (m_center+mid) ;
	if ( bFlag )
		vertexArray.append (m_center+mid/2) ;

	return (Acad::eOk) ;
}

//Acad::ErrorStatus Asdk2dSector::getGripPoints (
//	AcGePoint3dArray &gripPoints,
//	AcDbIntArray &osnapModes,
//	AcDbIntArray &geomIds
//) const {
//	assertReadEnabled () ;
//	return (getVertices (gripPoints, true)) ;
//}
//
//Acad::ErrorStatus Asdk2dSector::moveGripPointsAt (
//	const AcDbIntArray &indices,
//	const AcGeVector3d &offset
//) {
//	assertWriteEnabled () ;
//	AcGeMatrix3d mat ;
//	AcGeVector3d v1, v2, v3 ;
//	AcGePoint3dArray gripPoints ;
//
//	getVertices (gripPoints, true) ;
//	for ( int i =0 ; i < indices.length () ; i++ ) {
//		switch ( indices [i] ) {
//			case 0: //----- Move center point
//				mat.entry [0] [3] =offset [X] ;
//				mat.entry [1] [3] =offset [Y] ;
//				mat.entry [2] [3] =offset [Z] ;
//				this->transformBy (mat) ;
//				break ;
//			case 1: //----- Increase both start angle, and radius
//				v1 =gripPoints [1] - m_center + offset ;
//				m_radius =v1.length () ;
//			case 2: //----- Increase start angle
//				v1 =gripPoints [indices [i]] - m_center ;
//				v2 =v1 + offset ;
//				v3 =m_normal.crossProduct (v1) ;
//				if ( v2.dotProduct (v3) < 0 )
//					m_start -=v1.angleTo (v2, v3) ;
//				else
//					m_start +=v1.angleTo (v2, v3) ;
//				if ( m_start >= 2*PI )
//					m_start -=2*PI ;
//				else if ( m_start < 0 )
//					m_start +=2*PI ;
//				break ;
//			case 3: //----- Increase both stop angle, and radius
//				v1 =gripPoints [3] - m_center + offset ;
//				m_radius =v1.length () ;
//			case 4: //----- Increase stop angle
//				v1 =gripPoints [indices [i]] - m_center ;
//				v2 =v1 + offset ;
//				v3 =m_normal.crossProduct (v1) ;
//				if ( v2.dotProduct (v3) < 0 )
//					m_stop -=v1.angleTo (v2, v3) ;
//				else
//					m_stop +=v1.angleTo (v2, v3) ;
//				if ( m_stop >= 2*PI )
//					m_stop -=2*PI ;
//				else if ( m_stop < 0 )
//					m_stop +=2*PI ;
//				break ;
//			case 5: //----- Increase radius
//				v1 =gripPoints [5] - m_center + offset ;
//				m_radius =v1.length () ;
//				break ;
//			case 6: //----- Rotate sector
//				v1 =gripPoints [6] - m_center ;
//				v2 =v1 + offset ;
//				v3 =m_normal.crossProduct (v1) ;
//				if ( v2.dotProduct (v3) < 0 ) {
//					m_start -=v1.angleTo (v2, v3) ;
//					m_stop -=v1.angleTo (v2, v3) ;
//				} else {
//					m_start +=v1.angleTo (v2, v3) ;
//					m_stop +=v1.angleTo (v2, v3) ;
//				}
//				if ( m_start >= 2*PI )
//					m_start -=2*PI ;
//				else if ( m_start < 0 )
//					m_start +=2*PI ;
//				if ( m_stop >= 2*PI )
//					m_stop -=2*PI ;
//				else if ( m_stop < 0 )
//					m_stop +=2*PI ;
//				break ;
//			default:
//				break ;
//		}
//	}
//	return (Acad::eOk) ;
//}
//
//Acad::ErrorStatus Asdk2dSector::getOsnapPoints (
//	AcDb::OsnapMode osnapMode,
//	int gsSelectionMark,
//	const AcGePoint3d &pickPoint,
//	const AcGePoint3d &lastPoint,
//	const AcGeMatrix3d &viewXform,
//	AcGePoint3dArray &snapPoints,
//	AcDbIntArray &geomIds
//) const {
//	assertReadEnabled () ;
//
//	AcGePoint3dArray vertexArray ;
//    getVertices (vertexArray, false) ;
//	switch ( osnapMode ) {
//		case AcDb::kOsModeEnd:
//			snapPoints.append (vertexArray [0]) ;
//			snapPoints.append (vertexArray [1]) ;
//			snapPoints.append (vertexArray [3]) ;
//			break;
//		case AcDb::kOsModeMid:
//			snapPoints.append (vertexArray [2]) ;
//			snapPoints.append (vertexArray [4]) ;
//			snapPoints.append (vertexArray [5]) ;
//			break ;
//		case AcDb::kOsModeCen:
//			snapPoints.append (vertexArray [0]) ;
//			break;
//		case AcDb::kOsModeNode:
//			//----- todo: ???
//			break ;
//		case AcDb::kOsModeQuad:
//			//----- todo:
//			break ;
//		case AcDb::kOsModeIns:
//			snapPoints.append (vertexArray [0]) ;
//			break ;
//		case AcDb::kOsModePerp:
//			//----- todo:
//			break;
//		case AcDb::kOsModeTan:
//			//----- todo:
//			break ;
//		case AcDb::kOsModeNear:
//			//----- todo:
//			break;
//		default:
//			return (Acad::eOk) ;
//	}
//	return (Acad::eOk) ;
//}

//------ Default is xxxGripPoints ()
//Acad::ErrorStatus Asdk2dSector::getStretchPoints (AcGePoint3dArray &stretchPoints) const {
//	assertReadEnabled () ;
//
//	return (Acad::eOk) ;
//}

//------ Default is transformBy ()
//Acad::ErrorStatus Asdk2dSector::moveStretchPoints (const AcDbIntArray &indices, const AcGeVector3d &offset) {
//	assertWriteEnabled () ;
//
//	return (Acad::eOk) ;
//}

Acad::ErrorStatus Asdk2dSector::dxfOutFields (AcDbDxfFiler *pFiler) const {
	assertReadEnabled () ;
	AcDbEntity::dxfOutFields (pFiler) ;
	//----- 100 SubClass
    pFiler->writeItem (AcDb::kDxfSubclass, ACRX_T("Asdk2dSector")) ;
	//----- 10 Center
	pFiler->writePoint3d (AcDb::kDxfXCoord, m_center) ;
	//----- 40 Radius
	pFiler->writeDouble (AcDb::kDxfReal, m_radius) ;
	//----- 41 Start Angle
	pFiler->writeDouble (AcDb::kDxfReal+1, m_start) ;
	//----- 42 Stop Angle
	pFiler->writeDouble (AcDb::kDxfReal+2, m_stop) ;
	//----- 210 Normal
	pFiler->writeItem (AcDb::kDxfNormalX, m_normal) ;
	return (Acad::eOk) ;
}

Acad::ErrorStatus Asdk2dSector::dxfInFields (AcDbDxfFiler *pFiler) {
	assertWriteEnabled () ;
	struct resbuf rb ;

	AcDbEntity::dxfInFields (pFiler) ;
    if ( pFiler->atSubclassData (ACRX_T("Asdk2dSector")) == false ) { //----- 100
        return (pFiler->filerStatus ()) ;
    }

	while ( pFiler->readResBuf (&rb) == Acad::eOk ) {
		switch ( rb.restype ) {
			case AcDb::kDxfXCoord: //----- 10
				m_center =asPnt3d (rb.resval.rpoint) ;
				break ;
			case AcDb::kDxfReal: //----- 40
				m_radius =rb.resval.rreal ;
				break ;
			case AcDb::kDxfReal+1: //----- 41
				m_start =rb.resval.rreal ;
				break ;
			case AcDb::kDxfReal+2: //----- 42
				m_stop =rb.resval.rreal ;
				break ;
			case AcDb::kDxfNormalX: //----- 210
				m_normal =asVec3d (rb.resval.rpoint) ;
				break ;
			default:
				pFiler->pushBackItem () ;
				break ;
		}
	}
    return (Acad::eOk) ;
}

Acad::ErrorStatus Asdk2dSector::dwgOutFields (AcDbDwgFiler *pFiler) const {
	assertReadEnabled () ;
	AcDbEntity::dwgOutFields (pFiler) ;
    pFiler->writeItem (m_center) ;
    pFiler->writeItem (m_radius) ;
	pFiler->writeItem (m_start) ;
	pFiler->writeItem (m_stop) ;
    pFiler->writeItem (m_normal) ;
	return (Acad::eOk) ;
}

Acad::ErrorStatus Asdk2dSector::dwgInFields (AcDbDwgFiler *pFiler) {
	assertWriteEnabled () ;
	AcDbEntity::dwgInFields (pFiler) ;
	pFiler->readItem (&m_center) ;
	pFiler->readItem (&m_radius) ;
	pFiler->readItem (&m_start) ;
	pFiler->readItem (&m_stop) ;
	pFiler->readItem (&m_normal) ;
	return (Acad::eOk) ;
}

//Acad::ErrorStatus Asdk2dSector::transformBy (const AcGeMatrix3d &xform) {
//	if ( !xform.isUniScaledOrtho () )
//		return (Acad::eCannotScaleNonUniformly) ;
//	assertWriteEnabled () ;
//
//	AcGePoint3d origin ;
//	origin.transformBy (xform) ;
//	AcGePoint3d start (cos (m_start), sin (m_start), 0) ;
//	ucs2wcs (m_normal, start) ;
//	start.transformBy (xform) ;
//	AcGePoint3d stop (cos (m_stop), sin(m_stop), 0) ;
//	ucs2wcs (m_normal, stop) ;
//	stop.transformBy (xform) ;
//
//	m_center.transformBy (xform) ;
//	m_radius =xformDbl (xform, m_radius) ;
//	m_normal.transformBy (xform) ;
//	m_normal.normalize () ;
//
//	start -=origin.asVector () ;
//	wcs2ucs (m_normal, start) ;
//	m_start =atan2 (start [Y], start [X]) ;
//	stop -=origin.asVector () ;
//	wcs2ucs (m_normal, stop) ;
//	m_stop =atan2 (stop [Y], stop [X]) ;
//
//	//----- Mirroring xform, switch start & end angles
//	if ( xform.det () < 0 ) {
//		double tmp =m_start ;
//		m_start =m_stop ;
//		m_stop =tmp ;
//	}
//
////	if ( fabs (thickness ()) > 1e-10 )
////		setThickness (xformDbl (xform, thickness ())) ;
//	return (Acad::eOk) ;
//}
//
//Adesk::Boolean Asdk2dSector::worldDraw (AcGiWorldDraw *mode) {
//	mode->geometry ().circle (m_center, m_radius, AcGeVector3d (0.0, 0.0, 1.0)) ;
//	return true ;
//}

void Asdk2dSector::saveAs (AcGiWorldDraw* mode, AcDb::SaveType st)
{
}